;Example POOL program, you may use it,
;modify and distribute under GPL license.
;Authors: R&D Independent Team
;Source: www.pool-rnd.com

;base class for turtles drawing the fractal
to painter
  ;prototype function for the fractal formula
  to formula :real :img :depth output 0 end
  ;prototype function for color scaling
  to fcolor :idx output 0 end
  ;prototype function that precalculates color scaling factor
  to cscale output 1 end
  ;prototype function that calculates iteration limit
  to depth :s output 1 end

  to fxy :x :y
    let "c round :cscale * fcolor formula :x :y :depth
    ifelse :c >= 1 && :c < 1000 [op :c] [op {0 0 0 0}]
  end

  ;repaint window content
  to do_render :walls :depth
    "depth := depth :depth
    "cscale := cscale
    while not :break [
      lock "lines [ ;take the next line to render
        ifelse count :lines > 0
        [let "y dequeue :lines]
        [break]
      ]
      :walls,3 := :y
      :walls,4 := :y
      (render $fxy 100 :walls)
    ]
    signal "tstop
  end

  let "cscale 1 ;scale from value at the point to its color
  pu ht
end
;--- end of painter class ---

;class of a turtle that is launching calculations
;and measures the time
to tmeasure
  to start :data
    wait 30 ;wait for frequently repeated calls (e.g. scroll)
    if not (:break || empty? :t) [
      (print "|start render...|)
      let "ms timemilli
      lock "t [(do_render :data,1 :data,2) @ :t]
      (waitsignal "tstop :t)
      (print "|render time:| 0.001 * (timemilli - :ms))
    ]
    signal "ready
  end
  pu ht signal "ready
end
;--- end of tmeasure class ---

shared [t break lines]
"lines := [] ;list of actual Y coordinates of the image lines
"t := [] ;list of turtles painting the fractal
"break := false
ht pu

;turtle to start calculations and measure the time
"u := (newt $tmeasure)

;event handler for the window zoom or position changes
;(repaints the window content)
to onworldchanged :walls
  if empty? :t [stop]

  "break := true
  (waitsignal "ready :u)
  "break := false

  let "ymax :walls,3
  let "ymin :walls,4
  let "dy 1 / :pool_cfg,"scale_y
  lock "lines [
    "lines := []
    for [y [:ymax + :dy] :ymin -:dy] [queue :lines :y]
  ]

  cs (start array :walls :pool_cfg,"scale_x) @ :u
end

